@@ -184,8 +184,8 @@ pub enum Subs {
         tags: Option<TagFilter>,
         #[structopt(long, help = "Create script without invoking the editor")]
         fast: bool,
-        #[structopt(parse(try_from_str), default_value = ".")]
-        edit_query: EditQuery,
+        #[structopt(parse(try_from_str), default_value = "?")]
+        edit_query: EditQuery<ScriptQuery>,
         content: Vec<String>,
     },
     #[structopt(about = "Manage alias", settings = NO_FLAG_SETTINGS)]
@@ -212,6 +212,13 @@ pub enum Subs {
         repeat: Option<u64>,
         #[structopt(long, short, help = "Use arguments from last run")]
         previous_args: bool,
+        #[structopt(
+            long,
+            short = "E",
+            requires = "previous-args",
+            help = "Raise an error if --previous-args is given but there is no previous argument"
+        )]
+        error_no_previous: bool,
         #[structopt(long, short, requires = "previous-args")]
         dir: Option<PathBuf>,
         #[structopt(default_value = "-", parse(try_from_str))]
@@ -248,7 +255,7 @@ pub enum Subs {
         #[structopt(long, short)]
         tags: Option<TagFilter>,
         origin: ScriptQuery,
-        new: ScriptName,
+        new: EditQuery<ScriptName>,
     },
     #[structopt(about = "Move the script to another one")]
     MV {
@@ -263,7 +270,7 @@ pub enum Subs {
         tags: Option<TagFilter>,
         #[structopt(parse(try_from_str))]
         origin: ListQuery,
-        new: Option<ScriptName>,
+        new: Option<EditQuery<ScriptName>>,
     },
     #[structopt(
         about = "Manage script tags. If a tag filter is given, store it to config, otherwise show tag information."
@@ -337,8 +344,8 @@ pub struct List {
     pub queries: Vec<ListQuery>,
 }
 
-fn set_home(p: &Option<String>) -> Result {
-    path::set_home(p.as_ref())?;
+fn set_home(p: &Option<String>, create_on_missing: bool) -> Result {
+    path::set_home(p.as_ref(), create_on_missing)?;
     Config::init()
 }
 
@@ -370,7 +377,7 @@ fn handle_alias_args(args: Vec<String>) -> Result<Root> {
     match AliasRoot::from_iter_safe(&args) {
         Ok(alias_root) => {
             log::info!("別名命令行物件 {:?}", alias_root);
-            set_home(&alias_root.root_args.hs_home)?;
+            set_home(&alias_root.root_args.hs_home, true)?;
             let mut root = match alias_root.expand_alias(&args, Config::get()) {
                 Some(new_args) => Root::from_iter(new_args),
                 None => Root::from_iter(&args),
@@ -389,9 +396,9 @@ fn handle_alias_args(args: Vec<String>) -> Result<Root> {
 impl Root {
     /// 若帶了 --no-alias 選項，或是補全模式，我們可以把設定腳本之家（以及載入設定檔）的時間再推遲
     /// 在補全模式中意義重大，因為使用者可能會用 -H 指定別的腳本之家
-    pub fn set_home_unless_from_alias(&self) -> Result {
+    pub fn set_home_unless_from_alias(&self, create_on_missing: bool) -> Result {
         if !self.is_from_alias {
-            set_home(&self.root_args.hs_home)?;
+            set_home(&self.root_args.hs_home, create_on_missing)?;
         }
         Ok(())
     }
@@ -421,7 +428,7 @@ impl Root {
             None => {
                 log::info!("無參數模式");
                 self.subcmd = Some(Subs::Edit {
-                    edit_query: EditQuery::default(),
+                    edit_query: EditQuery::Query(Default::default()),
                     ty: None,
                     content: vec![],
                     tags: None,
@@ -527,6 +534,7 @@ mod test {
             Some(Subs::Run {
                 dummy: true,
                 previous_args: false,
+                error_no_previous: false,
                 repeat: Some(42),
                 dir: None,
                 script_query,
