@@ -1,24 +1,23 @@
 use fxhash::FxHashMap as HashMap;
-use hyper_scripter::args::{
-    self, History, List, Root, RootArgs, Subs, Tags, TagsSubs, Types, TypesSubs,
-};
+use hyper_scripter::args::{self, History, List, Root, Subs, Tags, TagsSubs, Types, TypesSubs};
 use hyper_scripter::config::{Config, NamedTagFilter};
 use hyper_scripter::error::{Error, RedundantOpt, Result};
 use hyper_scripter::extract_msg::{extract_env_from_content, extract_help_from_content};
 use hyper_scripter::list::{fmt_list, DisplayIdentStyle, DisplayStyle, ListOptions};
 use hyper_scripter::path;
-use hyper_scripter::query::{self, ScriptQuery};
+use hyper_scripter::query::{self, EditQuery, ScriptQuery};
 use hyper_scripter::script_repo::{RepoEntry, ScriptRepo};
 use hyper_scripter::script_time::ScriptTime;
 use hyper_scripter::tag::{Tag, TagFilter};
 use hyper_scripter::util::{
     self, completion_util,
+    holder::RepoHolder,
     main_util::{self, EditTagArgs},
     print_iter,
 };
 use hyper_scripter::Either;
 use hyper_scripter::{db, migration};
-use hyper_scripter_historian::{Historian, IgnoreResult};
+use hyper_scripter_historian::{Historian, LastTimeRecord};
 
 #[tokio::main]
 async fn main() {
@@ -58,7 +57,7 @@ async fn main_err_handle() -> Result<Vec<Error>> {
         return Ok(vec![]);
     }
 
-    root.set_home_unless_from_alias()?;
+    root.set_home_unless_from_alias(true)?;
 
     if matches!(root.subcmd, Some(Subs::Migrate)) {
         migration::do_migrate(db::get_file()).await?;
@@ -82,19 +81,6 @@ struct MainReturn {
     /// 用來裝那種不會馬上造成中止的錯誤，例如 ScriptError
     errs: Vec<Error>,
 }
-struct RepoHolder {
-    root_args: RootArgs,
-    need_journal: bool,
-}
-impl RepoHolder {
-    async fn init(self) -> Result<ScriptRepo> {
-        util::init_repo(self.root_args, self.need_journal).await
-    }
-    async fn historian(self) -> Result<Historian> {
-        let historian = Historian::new(path::get_home().to_owned()).await?;
-        Ok(historian)
-    }
-}
 
 async fn main_inner(root: Root) -> Result<MainReturn> {
     Config::set_prompt_level(root.root_args.prompt_level);
@@ -120,7 +106,11 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
     }
 
     match root.subcmd.unwrap() {
-        Subs::LoadUtils => main_util::load_utils(&mut repo.init().await?).await?,
+        Subs::LoadUtils => {
+            let (mut repo, closer) = repo.init().await?;
+            main_util::load_utils(&mut repo).await?;
+            closer.close(repo).await;
+        }
         Subs::Alias {
             unset: false,
             before: Some(before),
@@ -179,7 +169,7 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
             no_template,
         } => {
             // TODO: 這裡邏輯太複雜了，抽出來測試吧
-            let mut repo = repo.init().await?;
+            let (mut repo, closer) = repo.init().await?;
             let edit_tags = {
                 // TODO: 不要這麼愛 clone
                 let mut innate_tags = conf.main_tag_filter.clone();
@@ -204,7 +194,7 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
             let (path, mut entry) =
                 main_util::edit_or_create(edit_query, &mut repo, ty, edit_tags).await?;
             let prepare_resp = util::prepare_script(&path, &*entry, no_template, &content)?;
-            entry.update(|info| info.read()).await?;
+            create_read_event(&mut entry).await?;
             if !fast {
                 let res = util::open_editor(&path);
                 if let Err(err) = res {
@@ -217,9 +207,10 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
                 let id = entry.id;
                 repo.remove(id).await?
             }
+            closer.close(repo).await;
         }
         Subs::Help { args } => {
-            let mut repo = repo.init().await?;
+            let (mut repo, closer) = repo.init().await?;
             let script_query: ScriptQuery = args[0].parse()?;
             let mut entry = query::do_script_query_strict(&script_query, &mut repo).await?;
             log::info!("檢視用法： {:?}", entry.name);
@@ -237,16 +228,18 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
                 }
                 print_iter(envs, "\n");
             }
+            closer.close(repo).await;
         }
         Subs::Run {
             script_query,
             dummy,
             args,
             previous_args,
+            error_no_previous,
             repeat,
             dir,
         } => {
-            let mut repo = repo.init().await?;
+            let (mut repo, closer) = repo.init().await?;
             let mut entry = query::do_script_query_strict(&script_query, &mut repo).await?;
             main_util::run_n_times(
                 repeat.unwrap_or(1),
@@ -255,29 +248,33 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
                 args,
                 &mut ret.errs,
                 previous_args,
+                error_no_previous,
                 dir,
             )
             .await?;
+            closer.close(repo).await;
         }
         Subs::Which { script_query } => {
-            let mut repo = repo.init().await?;
+            let (mut repo, closer) = repo.init().await?;
             let entry = query::do_script_query_strict(&script_query, &mut repo).await?;
             log::info!("定位 {:?}", entry.name);
             // NOTE: 不檢查存在與否
             let p = path::get_home().join(entry.file_path_fallback());
             println!("{}", p.to_string_lossy());
+            closer.close(repo).await;
         }
         Subs::Cat { script_query } => {
-            let mut repo = repo.init().await?;
+            let (mut repo, closer) = repo.init().await?;
             let mut entry = query::do_script_query_strict(&script_query, &mut repo).await?;
             log::info!("打印 {:?}", entry.name);
             let script_path = path::open_script(&entry.name, &entry.ty, Some(true))?;
             let content = util::read_file(&script_path)?;
             print!("{}", content);
             create_read_event(&mut entry).await?;
+            closer.close(repo).await;
         }
         Subs::EnvHelp { script_query } => {
-            let mut repo = repo.init().await?;
+            let (mut repo, closer) = repo.init().await?;
             let mut entry = query::do_script_query_strict(&script_query, &mut repo).await?;
             log::info!("打印 {:?} 的環境變數", entry.name);
             create_read_event(&mut entry).await?;
@@ -285,6 +282,7 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
             let content = util::read_file(&script_path)?;
             let envs = extract_env_from_content(&content);
             print_iter(envs, "\n");
+            closer.close(repo).await;
         }
         Subs::Types(Types {
             subcmd: Some(TypesSubs::LS),
@@ -325,10 +323,12 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
                 display_style,
             };
             let stdout = std::io::stdout();
-            fmt_list(&mut stdout.lock(), &mut repo.init().await?, opt).await?;
+            let (mut repo, closer) = repo.init().await?;
+            fmt_list(&mut stdout.lock(), &mut repo, opt).await?;
+            closer.close(repo).await;
         }
         Subs::RM { queries, purge } => {
-            let mut repo = repo.init().await?;
+            let (mut repo, closer) = repo.init().await?;
             let delete_tag: Option<TagFilter> = Some("+removed".parse().unwrap());
             let mut to_purge = vec![]; // (name, ty, id)
             for mut entry in query::do_list_query(&mut repo, &queries).await?.into_iter() {
@@ -356,30 +356,34 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
                     log::warn!("刪除腳本實體遭遇錯誤：{}", e);
                 }
             }
+            closer.close(repo).await;
         }
         Subs::CP { origin, new, tags } => {
-            let mut repo = repo.init().await?;
-            if repo.get_mut(&new, true).is_some() {
-                return Err(Error::ScriptExist(new.to_string()));
+            let (mut repo, closer) = repo.init().await?;
+            if let EditQuery::Query(new) = &new {
+                if repo.get_mut(new, true).is_some() {
+                    return Err(Error::ScriptExist(new.to_string()));
+                }
             }
             let entry = query::do_script_query_strict(&origin, &mut repo).await?;
             let og_script = path::open_script(&entry.name, &entry.ty, Some(true))?;
-            let new_script = path::open_script(&new, &entry.ty, Some(false))?;
-            util::cp(&og_script, &new_script)?;
-            let mut new_info = entry.cp(new.clone());
+            let (new_name, new_path) = match new {
+                EditQuery::NewAnonimous => path::open_new_anonymous(&entry.ty)?,
+                EditQuery::Query(new) => {
+                    let new_path = path::open_script(&new, &entry.ty, Some(false))?;
+                    (new, new_path)
+                }
+            };
+            util::cp(&og_script, &new_path)?;
+            let mut new_info = entry.cp(new_name);
 
             if let Some(tags) = tags {
-                // TODO: delete tag
-                if tags.append {
-                    log::debug!("附加上標籤：{:?}", tags);
-                    new_info.tags.extend(tags.into_allowed_iter());
-                } else {
-                    log::debug!("設定標籤：{:?}", tags);
-                    new_info.tags = tags.into_allowed_iter().collect();
-                }
+                new_info.append_tags(tags);
             }
 
-            repo.entry(&new).or_insert(new_info).await?;
+            let mut entry = repo.entry(&new_info.name).or_insert(new_info).await?;
+            create_read_event(&mut entry).await?; //FIXME: 一旦可以 left join 就省掉這個
+            closer.close(repo).await;
         }
         Subs::MV {
             origin,
@@ -387,14 +391,15 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
             tags,
             ty,
         } => {
-            let mut repo = repo.init().await?;
+            let (mut repo, closer) = repo.init().await?;
             let new_name = match new {
-                Some(name) => {
+                Some(EditQuery::Query(name)) => {
                     if repo.get_mut(&name, true).is_some() {
                         return Err(Error::ScriptExist(name.to_string()));
                     }
                     Some(name)
                 }
+                Some(EditQuery::NewAnonimous) => Some(path::new_anonymous_name()?),
                 None => None,
             };
             let mut scripts = query::do_list_query(&mut repo, &[origin]).await?;
@@ -407,12 +412,13 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
                     .into());
                 }
                 //  只有一個，放心移動
-                main_util::mv(&mut scripts[0], new_name, ty.clone(), tags.clone()).await?;
+                main_util::mv(&mut scripts[0], new_name, ty.clone(), tags).await?;
             } else {
                 for entry in scripts.iter_mut() {
                     main_util::mv(entry, None, ty.clone(), tags.clone()).await?;
                 }
             }
+            closer.close(repo).await;
         }
         Subs::Tags(Tags {
             subcmd: Some(TagsSubs::Unset { name }),
@@ -458,7 +464,7 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
                     known,
                 }),
         }) => {
-            let mut repo = repo.init().await?;
+            let (mut repo, closer) = repo.init().await?;
             if known {
                 print_iter(known_tags_iter(&mut repo), " ");
             } else {
@@ -484,6 +490,7 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
                 }
                 println!();
             }
+            closer.close(repo).await;
         }
         Subs::Tags(Tags {
             subcmd: Some(TagsSubs::Set { content, name }),
@@ -511,7 +518,7 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
         Subs::History {
             subcmd: History::RM { queries, range },
         } => {
-            let mut repo = repo.init().await?;
+            let (mut repo, closer) = repo.init().await?;
             let historian = repo.historian().clone();
             let mut scripts = query::do_list_query(&mut repo, &queries).await?;
             let ids: Vec<_> = scripts.iter().map(|s| s.id).collect();
@@ -538,6 +545,7 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
                         .await?;
                 }
             }
+            closer.close(repo).await;
         }
         Subs::History {
             subcmd: History::RMID { event_id },
@@ -552,27 +560,30 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
         Subs::History {
             subcmd: History::Amend { event_id, args },
         } => {
-            let historian = repo.historian().await?;
+            let (historian, closer) = repo.historian().await?;
             let args = serde_json::to_string(&args)?;
-            historian.amend_args_by_id(event_id as i64, &args).await?
+            historian.amend_args_by_id(event_id as i64, &args).await?;
+            closer.close(historian).await;
         }
         Subs::History {
             subcmd: History::Tidy { queries },
         } => {
-            let mut repo = repo.init().await?;
+            let (mut repo, closer) = repo.init().await?;
             let historian = repo.historian().clone();
             for entry in query::do_list_query(&mut repo, &queries).await?.into_iter() {
                 historian.tidy(entry.id).await?;
             }
+            closer.close(repo).await;
         }
         Subs::History {
             subcmd: History::Neglect { queries },
         } => {
-            let mut repo = repo.init().await?;
+            let (mut repo, closer) = repo.init().await?;
             for entry in query::do_list_query(&mut repo, &queries).await?.into_iter() {
                 let id = entry.id;
                 entry.get_env().handle_neglect(id).await?;
             }
+            closer.close(repo).await;
         }
         Subs::History {
             subcmd:
@@ -584,7 +595,7 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
                     dir,
                 },
         } => {
-            let mut repo = repo.init().await?;
+            let (mut repo, closer) = repo.init().await?;
             let historian = repo.historian().clone();
             let dir = util::option_map_res(dir, |d| path::normalize_path(d))?;
             let scripts = query::do_list_query(&mut repo, &queries).await?;
@@ -596,7 +607,10 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
                 log::debug!("嘗試打印參數 {} {}", script_id, args);
                 let args: Vec<String> = serde_json::from_str(&args)?;
                 if with_name {
-                    let entry = get_mut_by_id(&mut repo, script_id)?;
+                    let entry = repo.get_mut_by_id(script_id).ok_or_else(|| {
+                        log::error!("史學家給的腳本 id 竟然在倉庫中找不到……");
+                        Error::ScriptNotFound(script_id.to_string())
+                    })?;
                     print!("{}", entry.name.key());
                     if !args.is_empty() {
                         print!(" ");
@@ -605,6 +619,7 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
                 print_iter(args.into_iter().map(|s| util::to_display_args(s)), " ");
                 println!("");
             }
+            closer.close(repo).await;
         }
         sub => unimplemented!("{:?}", sub),
     }
@@ -640,35 +655,21 @@ fn known_tags_iter<'a>(repo: &'a mut ScriptRepo) -> impl Iterator<Item = &'a Tag
 }
 
 async fn process_event_by_id(is_humble: bool, repo: RepoHolder, event_id: u64) -> Result {
-    let mut repo = repo.init().await?;
-    let historian = repo.historian();
+    let (env, closer) = repo.env().await?;
     let event_id = event_id as i64;
     let res = if is_humble {
-        historian.humble_args_by_id(event_id).await?
+        env.historian.humble_args_by_id(event_id).await?
     } else {
-        historian.ignore_args_by_id(event_id).await?
+        env.historian.ignore_args_by_id(event_id).await?
     };
     if let Some(res) = res {
-        let mut entry = get_mut_by_id(&mut repo, res.script_id)?;
-        entry
-            .update(|info| {
-                info.exec_time = res.exec_time.map(|t| ScriptTime::new(t));
-                info.exec_done_time = res.exec_done_time.map(|t| ScriptTime::new(t));
-                info.humble_time = res.humble_time;
-            })
-            .await?;
+        env.update_last_time_directly(res).await?;
     }
+    closer.close(env).await;
     Ok(())
 }
 
-fn get_mut_by_id(repo: &mut ScriptRepo, id: i64) -> Result<RepoEntry<'_>> {
-    repo.get_mut_by_id(id).ok_or_else(|| {
-        log::error!("史學家給的腳本 id 竟然在倉庫中找不到……");
-        Error::ScriptNotFound(id.to_string())
-    })
-}
-
-fn check_time_changed(entry: &RepoEntry<'_>, ignrore_res: &IgnoreResult) -> bool {
+fn check_time_changed(entry: &RepoEntry<'_>, ignrore_res: &LastTimeRecord) -> bool {
     let s_exec_time = entry.exec_time.as_ref().map(|t| **t);
     let s_exec_done_time = entry.exec_done_time.as_ref().map(|t| **t);
     (s_exec_time, s_exec_done_time, entry.humble_time)
