@@ -14,22 +14,14 @@ mod range_query;
 pub use range_query::*;
 
 #[derive(Debug, Eq, PartialEq, Serialize)]
-pub enum EditQuery {
+pub enum EditQuery<Q> {
     NewAnonimous,
-    Query(ScriptQuery),
+    Query(Q),
 }
-impl Default for EditQuery {
-    fn default() -> Self {
-        EditQuery::Query(ScriptQuery {
-            inner: ScriptQueryInner::Prev(1),
-            bang: false,
-        })
-    }
-}
-impl FromStr for EditQuery {
+impl<Q: FromStr<Err = Error>> FromStr for EditQuery<Q> {
     type Err = Error;
     fn from_str(s: &str) -> Result<Self> {
-        Ok(if s == "." {
+        Ok(if s == "?" {
             EditQuery::NewAnonimous
         } else {
             EditQuery::Query(s.parse()?)
@@ -78,6 +70,14 @@ pub struct ScriptQuery {
     inner: ScriptQueryInner,
     bang: bool,
 }
+impl Default for ScriptQuery {
+    fn default() -> Self {
+        ScriptQuery {
+            inner: ScriptQueryInner::Prev(1),
+            bang: false,
+        }
+    }
+}
 impl std::fmt::Display for ScriptQuery {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match &self.inner {
@@ -153,7 +153,7 @@ impl FromStr for ScriptQuery {
         } else if s.starts_with('^') {
             ScriptQueryInner::Prev(parse_prev(s)?)
         } else {
-            ScriptName::valid(s, true).context("模糊搜尋仍需符合腳本名格式！")?; // NOTE: 單純檢查用
+            ScriptName::valid(s, true, true).context("模糊搜尋仍需符合腳本名格式！")?; // NOTE: 單純檢查用
             ScriptQueryInner::Fuzz(s.to_owned())
         };
         Ok(ScriptQuery { inner, bang })
