@@ -3,7 +3,7 @@ use crate::config::Config;
 use crate::error::{Contextable, Error, RedundantOpt, Result};
 use crate::extract_msg::extract_env_from_content;
 use crate::path;
-use crate::query::{self, EditQuery};
+use crate::query::{self, EditQuery, ScriptQuery};
 use crate::script::{IntoScriptName, ScriptInfo, ScriptName};
 use crate::script_repo::{RepoEntry, ScriptRepo};
 use crate::script_type::{iter_default_templates, ScriptType};
@@ -52,14 +52,7 @@ pub async fn mv(
                 info.name = name;
             }
             if let Some(tags) = tags {
-                // TODO: delete tag
-                if tags.append {
-                    log::debug!("附加上標籤：{:?}", tags);
-                    tags.fill_allowed_map(&mut info.tags);
-                } else {
-                    log::debug!("設定標籤：{:?}", tags);
-                    info.tags = tags.into_allowed_iter().collect();
-                }
+                info.append_tags(tags);
             }
             info.write();
         })
@@ -68,7 +61,7 @@ pub async fn mv(
 }
 // XXX 到底幹嘛把新增和編輯的邏輯攪在一處呢…？
 pub async fn edit_or_create(
-    edit_query: EditQuery,
+    edit_query: EditQuery<ScriptQuery>,
     script_repo: &'_ mut ScriptRepo,
     ty: Option<ScriptType>,
     tags: EditTagArgs,
@@ -77,8 +70,9 @@ pub async fn edit_or_create(
     let mut new_namespaces: Vec<Tag> = vec![];
 
     let (script_name, script_path) = if let EditQuery::Query(query) = edit_query {
-        macro_rules! new_named {
-            () => {{
+        match query::do_script_query(&query, script_repo, false, false).await {
+            // TODO: 手動測試文件？
+            Err(Error::DontFuzz) | Ok(None) => {
                 if tags.explicit_filter {
                     return Err(RedundantOpt::Filter.into());
                 }
@@ -100,15 +94,14 @@ pub async fn edit_or_create(
                 let p = path::open_script(&name, &final_ty, None)
                     .context(format!("打開新命名腳本失敗：{:?}", name))?;
                 if p.exists() {
+                    if p.is_dir() {
+                        return Err(Error::PathExist(p).context("與目錄撞路徑"));
+                    }
+                    check_path_collision(&p, script_repo)?;
                     log::warn!("編輯野生腳本！");
                 }
                 (name, p)
-            }};
-        }
-
-        match query::do_script_query(&query, script_repo, false, false).await {
-            Err(Error::DontFuzz) => new_named!(), // TODO: 手動測試文件？
-            Ok(None) => new_named!(),
+            }
             Ok(Some(entry)) => {
                 if ty.is_some() {
                     return Err(RedundantOpt::Type.into());
@@ -245,6 +238,7 @@ pub async fn run_n_times(
     mut args: Vec<String>,
     res: &mut Vec<Error>,
     use_previous_args: bool,
+    error_no_previous: bool,
     dir: Option<PathBuf>,
 ) -> Result {
     log::info!("執行 {:?}", entry.name);
@@ -254,6 +248,9 @@ pub async fn run_n_times(
         let dir = super::option_map_res(dir, |d| path::normalize_path(d))?;
         let historian = &entry.get_env().historian;
         match historian.previous_args(entry.id, dir.as_deref()).await? {
+            None if error_no_previous => {
+                return Err(Error::NoPreviousArgs);
+            }
             None => log::warn!("無前一次參數，當作空的"),
             Some(arg_str) => {
                 log::debug!("撈到前一次呼叫的參數 {}", arg_str);
@@ -401,3 +398,13 @@ pub async fn after_script(
     }
     Ok(true)
 }
+
+fn check_path_collision(p: &Path, script_repo: &mut ScriptRepo) -> Result {
+    for script in script_repo.iter_mut(true) {
+        let script_p = path::open_script(&script.name, &script.ty, None)?;
+        if &script_p == p {
+            return Err(Error::PathExist(script_p).context("與既存腳本撞路徑"));
+        }
+    }
+    Ok(())
+}
