@@ -13,11 +13,12 @@ use std::path::{Path, PathBuf};
 use std::process::{Command, ExitStatus};
 
 pub mod completion_util;
+pub mod holder;
 pub mod main_util;
 pub mod serde;
 
 pub mod init_repo;
-pub use init_repo::init_repo;
+pub use init_repo::*;
 
 pub fn illegal_name(s: &str) -> bool {
     s.starts_with('-')
@@ -25,6 +26,8 @@ pub fn illegal_name(s: &str) -> bool {
         || s.contains("..")
         || s.contains(' ')
         || s.contains('!')
+        || s.contains('?')
+        || s.contains('=')
         || s.contains('/')
         || s.is_empty()
 }
@@ -228,8 +231,13 @@ pub fn prepare_script<T: AsRef<str>>(
             let template = get_or_create_tamplate(&script.ty, true)?;
             handle_fs_res(&[path], write_prepare_script(file, &template, &info))?;
         } else {
+            let mut first = true;
             for line in content {
-                writeln!(file, "{}", line)?;
+                if !first {
+                    writeln!(file, "")?;
+                }
+                first = false;
+                write!(file, "{}", line)?;
             }
         }
     } else {
