@@ -105,18 +105,19 @@ fn test_edit_existing_bang() {
     run!("e test -t hide | echo 躲貓貓").unwrap();
 
     // 當場變一個異步執行期出來。不要直接把測試函式寫成異步，否則 setup 中鎖的處理會出問題…
-    let mut rt = tokio::runtime::Runtime::new().unwrap();
+    let rt = tokio::runtime::Runtime::new().unwrap();
     rt.block_on(async {
         use hyper_scripter::error::{Error, RedundantOpt};
         use hyper_scripter::script_repo::ScriptRepo;
         use hyper_scripter::tag::{Tag, TagFilter};
-        use hyper_scripter::util::main_util::{edit_or_create, EditTagArgs};
-        use hyper_scripter_historian::Historian;
+        use hyper_scripter::util::{
+            init_env,
+            main_util::{edit_or_create, EditTagArgs},
+        };
 
         let mut repo = {
-            let historian = Historian::new(get_home().to_owned()).await.unwrap();
-            let (pool, _) = hyper_scripter::db::get_pool(&mut true).await.unwrap();
-            ScriptRepo::new(pool, None, historian, true).await.unwrap()
+            let (env, _) = init_env(true).await.unwrap();
+            ScriptRepo::new(None, env).await.unwrap()
         };
         repo.filter_by_tag(&"all,^hide".parse::<TagFilter>().unwrap().into());
 
@@ -206,7 +207,7 @@ fn test_multifuzz() {
     let t2 = ScriptTest::new("multifuzz/t2", None, None);
 
     // 當場變一個異步執行期出來。不要直接把測試函式寫成異步，否則 setup 中鎖的處理會出問題…
-    let mut rt = tokio::runtime::Runtime::new().unwrap();
+    let rt = tokio::runtime::Runtime::new().unwrap();
     let res = rt.block_on(async {
         fuzz("mult", [&t1, &t2, &pref].iter().map(|t| t.get_name()), SEP)
             .await
@@ -283,3 +284,16 @@ fn test_fuzz_end_with_slash() {
     run!("e illegal/").expect_err("不應創建以`/`結尾的腳本");
     run!("mv - illegal/").expect_err("不應創建以`/`結尾的腳本");
 }
+
+#[test]
+fn test_existing_path() {
+    println!("測試路徑衝突的邊角案例");
+    let _g = setup();
+
+    let _ = ScriptTest::new("dir/file", None, None);
+    run!("e =dir -T txt | echo 1").expect_err("與目錄撞路徑");
+    run!("e =dir/file.sh -T txt | echo 1").expect_err("與既存腳本撞路徑");
+    run!("e =dir/file.sh/file | echo 1").unwrap_err();
+
+    assert_ls_len(1, Some("all"), None);
+}
